# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
=======
Unicode
=======
"""
from saunter.po.webdriver.element import Element
from saunter.SeleniumWrapper import SeleniumWrapper as wrapper
from saunter.exceptions import ElementNotFound
from saunter.SaunterWebDriver import SaunterWebDriver

class Unicode(Element):
    """
    Base element class for Unicode fields
    """
    def __set__(self, obj, val):
        e = obj.driver.find_element_by_locator(self.locator)
        e.send_keys(val)

    def __get__(self, obj, cls=None):
        e = obj.driver.find_element_by_locator(self.locator)
        if e.tag_name in ["input", "textarea"]:
            return e.get_attribute("value")
        return e.text



# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import os.path
import re
import shutil
import time
from selenium import selenium
import saunter.ConfigWrapper
from saunter.exceptions import ElementNotFound, WindowNotFound

class SaunterSelenium(selenium):
    """
    Extending the Se-RC driver
    """
    def __init__(self, *args):
        super(SaunterSelenium, self).__init__(*args)
        
        self.cf = saunter.ConfigWrapper.ConfigWrapper().config
        
    def stop(self):
        super(SaunterSelenium, self).stop()
        self.running = False

    def do_command(self, verb, args):
        try:
            return super(SaunterSelenium, self).do_command(verb, args)
        except Exception, e:
            if (re.match("ERROR: Element .* not found", unicode(e))):
                raise ElementNotFound(e)
                
            if (re.match("ERROR: Could not find window with .*", unicode(e))):
                raise WindowNotFound(e)
            raise Exception(unicode(e))

    def click(self, locator, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).click(locator)
        
    def double_click(self, locator, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).double_click(locator)

    def check(self, locator, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).check(locator)

    def click_at(self, locator, coordString, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).click_at(locator, coordString)

    def context_menu(self, locator, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).context_menu(locator)

    def context_menu_at(self, locator, coordString, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).context_menu_at(locator, coordString)
        
    def double_click_at(self, locator, coordString, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).double_click_at(locator, coordString)

    def drag_and_drop(self, locator, movementsString, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).drag_and_drop(locator, movementsString)

    def drag_and_drop_to_object(self, locatorOfObjectToBeDragged, locatorOfDragDestinationObject, focus_before = True):
        if focus_before:
            self.focus(locatorOfObjectToBeDragged)
        super(SaunterSelenium, self).drag_and_drop_to_object(locatorOfObjectToBeDragged, locatorOfDragDestinationObject)
    
    def focus(self, locator):
        try:
            super(SaunterSelenium, self).focus(locator)
        except:
            pass
            
    def fire_event(self, locator, eventName, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).fire_event(locator, eventName)
        
    def mouse_over(self, locator, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).mouse_over(locator)
        
    def mouse_down(self, locator, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).mouse_down(locator)
        
    def mouse_down_right(self, locator, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).mouse_down_right(locator)
        
    def mouse_down_at(self, locator, coordString, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).mouse_down_at(locator, coordString)
        
    def mouse_down_right_at(self, locator, coordString, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).mouse_down_right_at(locator, coordString)
        
    def mouse_move(self, locator, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).mouse_move(locator)
        
    def mouse_move_at(self, locator, coordString, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).mouse_move_at(locator, coordString)

    def open(self, url, ignoreResponseCode=True):
        super(SaunterSelenium, self).open(url, ignoreResponseCode=True)

    def open_window(self, url, windowID):
        super(SaunterSelenium, self).open_window(url, windowID)

    def select(self, selectLocator, optionLocator, focus_before = True):
        if focus_before:
            self.focus(selectLocator)
        super(SaunterSelenium, self).select(selectLocator, optionLocator)
        
    def add_selection(self, locator, optionLocator, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).add_selection(selectLocator, optionLocator)
        
    def remove_selection(self, locator, optionLocator, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).remove_selection(selectLocator, optionLocator)
        
    def remove_all_selections(self, locator, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).remove_all_selections(locator)

    def submit(self, formLocator):
        super(SaunterSelenium, self).submit(formLocator)

    def type(self, locator, value, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).type(locator, value)

    def type_keys(self, locator, value, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).type_keys(locator, value)

    def uncheck(self, locator, focus_before = True):
        if focus_before:
            self.focus(locator)
        super(SaunterSelenium, self).uncheck(locator)

    def wait_for_condition(self, script, timeout):
        super(SaunterSelenium, self).wait_for_condition(script, timeout)

    def wait_for_frame_to_load(self, frameAddress, timeout):
        super(SaunterSelenium, self).wait_for_frame_to_load(frameAddress, timeout)

    def wait_for_page_to_load(self, timeout):
        super(SaunterSelenium, self).wait_for_page_to_load(timeout)
        
    def wait_for_pop_up(self, windowID, timeout):
        super(SaunterSelenium, self).wait_for_pop_up(windowID, timeout)



import sys

import tailored.remotecontrol

class Matchers(object):
    def __init__(self, driver, verification_errors):
        self.driver = driver
        self.verification_erorrs = verification_errors

    #
    # unittest.TestCase provided
    #
    def assert_equal(self, first, second, msg=""):
        """
        Hard assert for equality

        :params first: the value to compare against
        :params second: the value to compare with
        :params msg: (Optional) msg explaining the difference
        """
        assert first == second

    def verify_equal(self, first, second, msg=""):
        """
        Soft assert for equality

        :params want: the value to compare against
        :params second: the value to compare with
        :params msg: (Optional) msg explaining the difference
        """
        try:
            self.assert_equal(first, second, msg)
        except AssertionError, e:
            if msg:
                m = "%s:\n%s" % (msg, str(e))
            else:
                m = str(e)
            self.verification_erorrs.append(m)

    def assert_not_equal(self, first, second, msg=""):
        """
        Hard assert for inequality

        :params want: the value to compare against
        :params secondv: the value to compare with
        :params msg: (Optional) msg explaining the difference
        """
        assert first != second

    def verify_not_equal(self, first, second, msg=""):
        """
        Soft assert for inequality

        :params want: the value to compare against
        :params second: the value to compare with
        :params msg: (Optional) msg explaining the difference
        """
        try:
            self.assert_not_equal(first, second, msg)
        except AssertionError, e:
            if msg:
                m = "%s:\n%s" % (msg, str(e))
            else:
                m = str(e)
            self.verification_erorrs.append(m)

    def assert_true(self, expr, msg=None):
        """
        Soft assert for whether the condition is true

        :params expr: the statement to evaluate
        :params msg: (Optional) msg explaining the difference
        """
        assert bool(expr) is True

    def verify_true(self, expr, msg=None):
        """
        Soft assert for whether the condition is true

        :params expr: the statement to evaluate
        :params msg: (Optional) msg explaining the difference
        """
        try:
            self.assert_true(expr, msg)
        except AssertionError, e:
            if msg:
                m = "%s:\n%s" % (msg, str(e))
            else:
                m = str(e)
            self.verification_erorrs.append(m)


    def assert_false(self, expr, msg=None):
        """
        Soft assert for whether the condition is false

        :params expr: the statement to evaluate
        :params msg: (Optional) msg explaining the difference
        """
        assert bool(expr) is False

    def verify_false(self, expr, msg=None):
        """
        Soft assert for whether the condition is false

        :params expr: the statement to evaluate
        :params msg: (Optional) msg explaining the difference
        """
        try:
            self.assert_false(expr, msg)
        except AssertionError, e:
            if msg:
                m = "%s:\n%s" % (msg, str(e))
            else:
                m = str(e)
            self.verification_erorrs.append(m)

    def assert_is(self, first, second, msg=None):
        """
        Hard assert for whether the parameters evaluate to the same object

        :params want: the object to compare against
        :params second: the object to compare with
        :params msg: (Optional) msg explaining the difference
        """
        assert first is second

    def verify_is(self, first, second, msg=None):
        """
        Soft assert for whether the parameters evaluate to the same object

        :params want: the object to compare against
        :params second: the object to compare with
        :params msg: (Optional) msg explaining the difference
        """
        try:
            self.assert_is(first, second, msg)
        except AssertionError, e:
            if msg:
                m = "%s:\n%s" % (msg, str(e))
            else:
                m = str(e)
            self.verification_erorrs.append(m)

    def assert_is_not(self, first, second, msg=None):
        """
        Hard assert for whether the parameters do not evaluate to the same object

        :params want: the object to compare against
        :params second: the object to compare with
        :params msg: (Optional) msg explaining the difference
        """
        assert first is not second

    def verify_is_not(self, first, second, msg=None):
        """
        Soft assert for whether the parameters do not evaluate to the same object

        :params want: the object to compare against
        :params second: the object to compare with
        :params msg: (Optional) msg explaining the difference
        """
        try:
            self.assert_is_not(first, second, msg)
        except AssertionError, e:
            if msg:
                m = "%s:\n%s" % (msg, str(e))
            else:
                m = str(e)
            self.verification_erorrs.append(m)

    def assert_is_none(self, expr, msg=None):
        """
        Hard assert for whether the expr is None

        :params expr: the expression to execute
        :params msg: (Optional) msg explaining the difference
        """
        assert expr is None

    def verify_is_none(self, expr, msg=None):
        """
        Soft assert for whether the expr is None

        :params want: the object to compare against
        :params second: the object to compare with
        :params msg: (Optional) msg explaining the difference
        """
        try:
            self.assert_is_none(expr, msg)
        except AssertionError, e:
            if msg:
                m = "%s:\n%s" % (msg, str(e))
            else:
                m = str(e)
            self.verification_erorrs.append(m)

    def assert_is_not_none(self, expr, msg=None):
        """
        Hard assert for whether the expr is not None

        :params expr: the expression to execute
        :params msg: (Optional) msg explaining the difference
        """
        assert expr is not None

    def verify_is_not_none(self, expr, msg=None):
        """
        Soft assert for whether the expr is not None

        :params want: the object to compare against
        :params second: the object to compare with
        :params msg: (Optional) msg explaining the difference
        """
        try:
            self.assert_is_not_none(expr, msg)
        except AssertionError, e:
            if msg:
                m = "%s:\n%s" % (msg, str(e))
            else:
                m = str(e)
            self.verification_erorrs.append(m)

    def assert_in(self, first, second, msg=""):
        """
        Hard assert for whether the first is in second

        :params first: the value to check
        :params second: the container to check in
        :params msg: (Optional) msg explaining the difference
        """
        assert first in second

    def verify_in(self, first, second, msg=""):
        """
        Soft assert for whether the first is in second

        :params first: the value to check
        :params second: the container to check in
        :params msg: (Optional) msg explaining the difference
        """
        try:
            self.assert_in(first, second, msg)
        except AssertionError, e:
            if msg:
                m = "%s:\n%s" % (msg, str(e))
            else:
                m = str(e)
            self.verification_erorrs.append(m)

    def assert_not_in(self, first, second, msg=""):
        """
        Hard assert for whether the first is not in second

        :params first: the value to check
        :params second: the container to check in
        :params msg: (Optional) msg explaining the difference
        """
        assert first not in second

    def verify_not_in(self, first, second, msg=""):
        """
        Soft assert for whether the first is not in second

        :params first: the value to check
        :params second: the container to check in
        :params msg: (Optional) msg explaining the difference
        """
        try:
            self.assert_not_in(first, second, msg)
        except AssertionError, e:
            if msg:
                m = "%s:\n%s" % (msg, str(e))
            else:
                m = str(e)
            self.verification_erorrs.append(m)

    def assert_is_instance(self, obj, cls, msg=""):
        """
        Hard assert for whether the is an instance of cls

        :params obj: the object instance
        :params cls: the class to compare against
        :params msg: (Optional) msg explaining the difference
        """
        assert isinstance(obj, cls)

    def verify_is_instance(self, obj, cls, msg=""):
        """
        Soft assert for whether the is an instance of cls

        :params obj: the object instance
        :params cls: the class to compare against
        :params msg: (Optional) msg explaining the difference
        """
        try:
            self.assert_is_instance(obj, cls, msg)
        except AssertionError, e:
            if msg:
                m = "%s:\n%s" % (msg, str(e))
            else:
                m = str(e)
            self.verification_erorrs.append(m)

    def assert_is_not_instance(self, obj, cls, msg=""):
        """
        Hard assert for whether the is not an instance of cls

        :params obj: the object instance
        :params cls: the class to compare against
        :params msg: (Optional) msg explaining the difference
        """
        assert not isinstance(obj, cls)

    def verify_is_not_instance(self, obj, cls, msg=""):
        """
        Soft assert for whether the is not an instance of cls

        :params obj: the object instance
        :params cls: the class to compare against
        :params msg: (Optional) msg explaining the difference
        """
        try:
            self.assert_is_not_instance(obj, cls, msg)
        except AssertionError, e:
            if msg:
                m = "%s:\n%s" % (msg, str(e))
            else:
                m = str(e)
            self.verification_erorrs.append(m)

    #
    # selenium specific
    #
    def assert_text_present(self, text, msg=None):    
        """
        Hard assert for whether the text if visible in the current window/frame

        :params text: the string to search for
        :params msg: (Optional) msg explaining the difference
        """
        if isinstance(self.driver, tailored.remotecontrol.RemoteControl):
            assert self.driver.is_text_present(text)
        else:
            e = driver.find_element_by_tag_name('body')
            assert text in e.text

    def verify_text_present(self, text, msg=None):
        """
        Soft assert for whether the text if visible in the current window/frame

        :params text: the string to search for
        :params msg: (Optional) msg explaining the difference
        """
        try:
            self.assert_text_present(text, msg)
        except AssertionError, e:
            if msg:
                m = "%s:\n%s" % (msg, str(e))
            else:
                m = str(e)
            self.verification_erorrs.append(m)

    def asset_element_present(self, locator, msg=None):
        """
        Hard assert for whether and element is present in the current window/frame

        :params locator: the locator of the element to search for
        :params msg: (Optional) msg explaining the difference
        """
        if isinstance(self.driver, tailored.remotecontrol.RemoteControl):
            assert self.driver.is_element_present(locator)
        else:
            e = driver.find_elements_by_locator(locator)
            if len(e) == 0:
                raise AssertionError("Element at %s was not found" % locator)

    def verify_element_present(self, locator, msg=None):
        """
        Soft assert for whether and element is present in the current window/frame

        :params locator: the locator of the element to search for
        :params msg: (Optional) msg explaining the difference
        """
        try:
            self.asset_element_present(locator, msg)
        except AssertionError, e:
            if msg:
                m = "%s:\n%s" % (msg, str(e))
            else:
                m = str(e)
            self.verification_erorrs.append(m)

    def assert_visible(self, locator, msg=None):
        """
        Hard assert for whether and element is present and visible in the current window/frame

        :params locator: the locator of the element to search for
        :params msg: (Optional) msg explaining the difference
        """
        if isinstance(self.driver, tailored.remotecontrol.RemoteControl):
            assert self.driver.is_visible(locator)
        else:
            e = driver.find_elements_by_locator(locator)
            if len(e) == 0:
                raise AssertionError("Element at %s was not found" % locator)
            assert e.is_displayed()
            
    def verify_visible(self, locator, msg=None):
        """
        Soft assert for whether and element is present and visible in the current window/frame

        :params locator: the locator of the element to search for
        :params msg: (Optional) msg explaining the difference
        """
        try:
            self.assert_visible(locator, msg)
        except AssertionError, e:
            if msg:
                m = "%s:\n%s" % (msg, str(e))
            else:
                m = str(e)
            self.verification_erorrs.append(m)



"""
=============
RemoteControl
=============
"""
from saunter.SaunterSelenium import SaunterSelenium

class RemoteControl(SaunterSelenium):
    """
    Modifications to the core Remote Control API are done in this class
    """
    def __init__(self, *args):
        super(RemoteControl, self).__init__(*args)


from _pytest.mark import MarkInfo
import json
import requests
import time
import os.path

class SauceLabs(object):
    def __init__(self, item):

        # session couldn't be established for some reason
        if not hasattr(item.parent._obj, "sauce_session"):
           return
        
        self.sauce_session = item.parent._obj.sauce_session
        self.username = item.parent._obj.config.get("SauceLabs", "username")
        self.key = item.parent._obj.config.get("SauceLabs", "key")
        self.log_dir = item.parent._obj.config.get("Saunter", "log_dir")

        j = {}

        # name
        j["name"] = item.name

        # result
        if item.outcome.lower() == 'passed':
            # print("pass")
            j["passed"] = True
        else:
            # print("fail")
            j["passed"] = False

        # tags
        j["tags"] = []
        j["custom-data"] = {}
        for keyword in item.keywords:
            if isinstance(item.keywords[keyword], MarkInfo):
                # per item custom data
                if keyword == "saucelabs_customdata":
                    for key, value in item.keywords[keyword].kwargs.iteritems():
                        j["custom-data"][key] = value
                # tags
                else:
                    j["tags"].append(keyword)

        # global custom data
        if item.parent._obj.config.has_section('SauceLabs CustomData'):
            for option in item.parent._obj.config.options('SauceLabs CustomData'):
                j["custom-data"][option] = item.parent._obj.config.get('SauceLabs CustomData', option)

        # print(json.dumps(j))

        # update
        which_url = "https://saucelabs.com/rest/v1/%s/jobs/%s" % (self.username, self.sauce_session)
        r = requests.put(which_url,
                         data=json.dumps(j),
                         headers={"Content-Type": "application/json"},
                         auth=(self.username, self.key))
        r.raise_for_status()

        if item.parent._obj.config.getboolean("SauceLabs", "get_video"):
            self._fetch_sauce_artifact("video.flv")

        if item.parent._obj.config.getboolean("SauceLabs", "get_log"):
            self._fetch_sauce_artifact("selenium-server.log")
        
    
    def _fetch_sauce_artifact(self, which):
        sauce_session = self.sauce_session
        which_url = "https://saucelabs.com/rest/%s/jobs/%s/results/%s" % (self.username, self.sauce_session, which)
        code = 404
        timeout = 0
        while code in [401, 404]:
            r = requests.get(which_url, auth = (self.username, self.key))
            try:
                code = r.status_code
                r.raise_for_status()
            except requests.exceptions.HTTPError, e:
                time.sleep(4)

        artifact = open(os.path.join(self.log_dir, which), "wb")
        artifact.write(r.content)
        artifact.close()


# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
======
Common
======
"""

import saunter.ConfigWrapper

cf = saunter.ConfigWrapper.ConfigWrapper().config
#: timeout value in s as an integer
if cf.has_option("Selenium", "timeout"):
    timeout_seconds = cf.getint("Selenium", "timeout")
else:
    timeout_seconds = 30
#: timout value in ms as an integer
timeout_microseconds = timeout_seconds * 1000
#: timout value in ms as a string
string_timeout = str(timeout_microseconds)


# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
================
sqlite3_provider
================
"""
import os.path
import sqlite3

import saunter.ConfigWrapper

class DBProvider(object):
    """
    SQLite3 powered provider

    :params db: name of db file located in support/db directory
    """
    def __init__(self, db):
        try:
            cf = saunter.ConfigWrapper.ConfigWrapper().config
        except:
            print('ooooo')
        self.db = sqlite3.connect(os.path.join(cf.get("Saunter", "base"), 'support', 'db', db))
        
    def __del__(self):
        self.db.close()
    
    def get_random_user(self):
        """
        Gets a random user from the provider

        :returns: Dictionary
        """
        c = self.db.cursor()
        c.execute('''SELECT username, password, fullname FROM users
                     WHERE rowid >= (abs(random()) % (SELECT max(rowid) FROM users))
                     LIMIT 1''')
        r = c.fetchone()
        return {"username": r[0], "password": r[1], "fullname": r[2]}


# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
==============
SyncExceptions
==============
"""
class SaunterExceptions(Exception):
    """Base class for exceptions in this module."""
    pass

class ElementNotFound(SaunterExceptions):
    def _get_message(self): 
        return self._message
    def _set_message(self, message): 
        self._message = message
    message = property(_get_message, _set_message)

class ElementVisiblityTimeout(SaunterExceptions):
    def _get_message(self): 
        return self._message
    def _set_message(self, message): 
        self._message = message
    message = property(_get_message, _set_message)

class ElementTextTimeout(SaunterExceptions):
    def _get_message(self): 
        return self._message
    def _set_message(self, message): 
        self._message = message
    message = property(_get_message, _set_message)
    
class InvalidLocatorString(SaunterExceptions):
    def _get_message(self): 
        return self._message
    def _set_message(self, message): 
        self._message = message
    message = property(_get_message, _set_message)

class WindowNotFound(SaunterExceptions):
    def _get_message(self): 
        return self._message
    def _set_message(self, message): 
        self._message = message
    message = property(_get_message, _set_message)


"""
=============
WebDriver
=============
"""
from saunter.SaunterWebDriver import SaunterWebDriver

class WebDriver(SaunterWebDriver):
    """
    Modifications to the core WebDriver API are done in this class
    """
    def __init__(self, **kwargs):
        super(WebDriver, self).__init__(**kwargs)




# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time
import urllib2
import os
import os.path
import saunter.matchers as matchers

class BaseTestCase(object):
    def assertEqual(self, first, second, msg=None):
        self.matchers.assert_equal(first, second, msg)

    def assertNotEqual(self, first, second, msg=None):
        self.matchers.assert_not_equal(first, second, msg)

    def assertTrue(self, expr, msg=None):
        self.matchers.assert_true(expr, msg)

    def assertFalse(self, expr, msg=None):
        self.matchers.assert_false(expr, msg)

    def assertIs(self, first, second, msg=None):
        self.matchers.assert_is(first, second, msg)

    def assertIsNot(self, first, second, msg=None):
        self.matchers.assert_is_not(first, second, msg)

    def assertIsNone(self, expr, msg=None):
        self.matchers.assert_is_none(expr, msg)

    def assertIsNotNone(self, expr, msg=None):
        self.matchers.assert_is_not_none(expr, msg)

    def assertIn(self, first, second, msg=None):
        self.matchers.assert_in(first, second, msg)

    def assertNotIn(self, first, second, msg=None):
        self.matchers.assert_not_in(first, second, msg)

    def assertIsInstance(self, obj, cls, msg=None):
        self.matchers.assert_is_instance(obj, cls, msg)

    def assertIsNotInstance(self, obj, cls, msg=None):
        self.matchers.assert_is_not_instance(obj, cls, msg)
    
    def _screenshot_prep_dirs(self):
        class_dir = os.path.join(os.path.join(self.config.get('Saunter', 'log_dir'), self.__class__.__name__))
        if not os.path.exists(class_dir):
            os.makedirs(class_dir)

        method_dir = os.path.join(class_dir, self.current_method_name)
        if not os.path.exists(method_dir):
            os.makedirs(method_dir)

        return method_dir
        


# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
===============
SaunterTestCase
===============
"""
import ConfigParser
import logging
import os

from saunter.SeleniumWrapper import SeleniumWrapper as wrapper

import saunter.ConfigWrapper
from saunter.matchers import Matchers

try:
    if saunter.ConfigWrapper.ConfigWrapper().config.getboolean("SauceLabs", "ondemand"):
        import json
except ConfigParser.NoSectionError as e:
    if "DOCGENERATION" not in os.environ:
        raise

from saunter.testcase.base import BaseTestCase

class SaunterTestCase(BaseTestCase):
    """
    Parent class of all script classes used for custom asserts (usually 'soft' asserts) and shared fixture setup
    and teardown
    """
    def setup_method(self, method):
        """
        Default setup method for all scripts. Connects either to the RC server configured in conf/selenium.ini
        or to Sauce Labs OnDemand
        """
        self.config = saunter.ConfigWrapper.ConfigWrapper().config
        self.cf = self.config

        self.current_method_name = method.__name__

        if self.cf.getboolean("SauceLabs", "ondemand"):
            host = self.cf.get("SauceLabs", "server_host")
            port = self.cf.get("SauceLabs", "server_port")
            j = {}
            j['username'] = self.cf.get("SauceLabs", "username")
            j['access-key'] = self.cf.get("SauceLabs", "key")
            j['os'] = self.cf.get("SauceLabs", "os")
            j['browser'] = self.cf.get("SauceLabs", "browser")
            if j['browser'][0] == "*":
                j['browser'] = j['browser'][1:]
            j['browser-version'] = self.cf.get("SauceLabs", "browser_version")
            browser = json.dumps(j)
        else:
            host = self.cf.get("Selenium", "server_host")
            port = self.cf.get("Selenium", "server_port")
            browser = self.cf.get("Selenium", "browser")

        self.selenium = wrapper().remote_control(host, port, browser, self.cf.get("Selenium", "base_url"))
        self.selenium.start()

        self.verificationErrors = []
        self.matchers = Matchers(self.selenium, self.verificationErrors)
        
        if self.cf.getboolean("SauceLabs", "ondemand"):
            self.sauce_session = self.selenium.get_eval("selenium.sessionId")
            
        
        self.selenium.window_maximize()
        if self.cf.has_option("Selenium", "timeout"):
            self.selenium.set_timeout(self.cf.getint("Selenium", "timeout") * 1000)
        self.selenium.open(self.cf.get("Selenium", "base_url"));

        self._screenshot_number = 1

    def teardown_method(self, method):
        if hasattr(self, "cf") and not self.cf.getboolean("SauceLabs", "ondemand"):
            self.take_named_screenshot("final")
        
        self.selenium.stop()

    def take_numbered_screenshot(self):
        if self.config.has_option("Saunter", "take_screenshots"):
            if self.cf.getboolean("Saunter", "take_screenshots"):
                method_dir = self._screenshot_prep_dirs()

                self.selenium.capture_screenshot(os.path.join(method_dir, str(self._screenshot_number).zfill(3) + ".png"))
                self._screenshot_number = self._screenshot_number + 1

    def take_named_screenshot(self, name):
        method_dir = self._screenshot_prep_dirs()

        self.selenium.capture_screenshot(os.path.join(method_dir, str(name) + ".png"))


# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
===============
django_provider
===============
"""
import os
import sys
import saunter.ConfigWrapper

class DjangoProvider(object):
    """
    Uses Django's Models to access the database
    """
    def __init__(self):
        cf = saunter.ConfigWrapper.ConfigWrapper().config
        django_where = cf.get("Django", "installation")
        if django_where not in sys.path:
            sys.path.append(django_where)

        django_name = cf.get("Django", "app")
        if not 'DJANGO_SETTINGS_MODULE' in os.environ:
            os.environ['DJANGO_SETTINGS_MODULE'] = "%s.settings" % django_name

    def get_random_user(self):
        """
        Gets a random user from the provider

        :returns: Dictionary
        """
        from provider.models import User
        u = User.objects.order_by('?')[0]
        return {"username": u.username, "password": u.password, "fullname": u.fullname}


# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
========
Checkbox
========
"""
from saunter.po.webdriver.element import Element

class CheckBox(Element):
    def __set__(self, obj, val):
        e = obj.driver.find_element_by_locator(self.locator)
        current = e.is_selected()
        if current != val:
            e.click()
            
    def __get__(self, obj, cls=None):
        e = obj.driver.find_element_by_locator(self.locator)
        return e.is_selected()


# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
======
Select
======
"""
from saunter.po.webdriver.element import Element
from saunter.web_element import WebElement
from saunter.SeleniumWrapper import SeleniumWrapper as wrapper
from saunter.exceptions import ElementNotFound
from saunter.SaunterWebDriver import SaunterWebDriver
from selenium.webdriver.support.select import Select as WebDriverSelect

class MultiSelect(Element, WebDriverSelect):
    def __get__(self, obj, cls=None):
        s = WebDriverSelect(obj.driver.find_element_by_locator(self.locator))
        a = s.all_selected_options
        if len(a) == 0:
            return None
        return [x.text for x in a]

    def __getitem__(self, key):
        s = WebDriverSelect(self.driver.find_element_by_locator(self.locator))
        a = s.all_selected_options
        if len(a) == 0:
            return None
        selections = [x.text for x in a]
        return selections[key]

    def __delitem__(self, key):
        s = WebDriverSelect(self.driver.find_element_by_locator(self.locator))
        method = key[:key.find("=")]
        value = key[key.find("=") + 1:]
        if method == "value":
            s.deselect_by_value(value)
        elif method == "index":
            s.deselect_by_index(value)
        elif method == "text":
            s.deselect_by_visible_text(value)
        else:
            raise saunter.exceptions.InvalidLocatorString("%s is an invalid locator" % item)

    def __len__(self):
        s = WebDriverSelect(self.driver.find_element_by_locator(self.locator))
        return len(s.all_selected_options)

    def append(self, item):
        s = WebDriverSelect(self.driver.find_element_by_locator(self.locator))
        method = item[:item.find("=")]
        value = item[item.find("=") + 1:]
        if method == "value":
            s.select_by_value(value)
        elif method == "index":
            s.select_by_index(value)
        elif method == "text":
            s.select_by_visible_text(value)
        else:
            raise saunter.exceptions.InvalidLocatorString("%s is an invalid locator" % item)



# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
======
Select
======
"""
from saunter.po.webdriver.element import Element
from saunter.web_element import WebElement
from saunter.SeleniumWrapper import SeleniumWrapper as wrapper
from saunter.exceptions import ElementNotFound
from saunter.SaunterWebDriver import SaunterWebDriver
from selenium.webdriver.support.select import Select as WebDriverSelect
import saunter.exceptions

class Select(Element, WebDriverSelect):
    """
    Base element class for Select fields
    """    
    def __set__(self, obj, val):
        s = WebDriverSelect(obj.driver.find_element_by_locator(self.locator))
        method = val[:val.find("=")]
        value = val[val.find("=") + 1:]
        if method == "value":
            s.select_by_value(value)
        elif method == "index":
            s.select_by_index(value)
        elif method == "text":
            s.select_by_visible_text(value)
        else:
            raise saunter.exceptions.InvalidLocatorString(val)
    
    def __get__(self, obj, cls=None):
        try:
            s = WebDriverSelect(obj.driver.find_element_by_locator(self.locator))
            e = s.first_selected_option
            return str(e.text)
        except AttributeError as e:
            if str(e) == "'SeleniumWrapper' object has no attribute 'connection'":
                pass
            else:
                raise e

class Select2(Element, WebDriverSelect):
    def __init__(self, driver, locator):
        self.driver = driver
        self.locator = locator
    
    @property
    def selected(self):
        s = WebDriverSelect(self.driver.find_element_by_locator(self.locator))
        e = s.first_selected_option
        return str(e.text)

    @selected.setter
    def selected(self, val):
        s = WebDriverSelect(self.driver.find_element_by_locator(self.locator))
        method = val[:val.find("=")]
        value = val[val.find("=") + 1:]
        if method == "value":
            s.select_by_value(value)
        elif method == "index":
            s.select_by_index(value)
        elif method == "text":
            s.select_by_visible_text(value)
        else:
            raise saunter.exceptions.InvalidLocatorString(val)

    @property
    def options(self):
        s = WebDriverSelect(self.driver.find_element_by_locator(self.locator))
        options = s.options
        text = [option.text.strip() for option in options]
        return text



from saunter.po.webdriver.page import Page as SaunterPage

from saunter.po import timeout_seconds
from selenium.webdriver.support.ui import WebDriverWait
from selenium.common.exceptions import TimeoutException
from selenium.common.exceptions import NoSuchElementException
import time

class Page(SaunterPage):
    pass    
        



# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
=============
ConfigWrapper
=============
"""
import ConfigParser
import os
import os.path

class ConfigWrapper(object):
    """
    Singleton reference to the config information
    """
    # singleton
    _instance = None
    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(ConfigWrapper, cls).__new__(cls, *args, **kwargs)
        return cls._instance

    def configure(self, config = "saunter.ini"):
        self.config = ConfigParser.SafeConfigParser()
        self.config.readfp(open(os.path.join("conf", config)))

# initialize the singleton
try:
    cf = ConfigWrapper().configure()
except IOError as e:
    if "DOCGENERATION" not in os.environ:
        raise


# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import os.path
import py
import sys
import random

def pytest_configure(config):
    sys.path.append(os.path.join(os.getcwd(), "modules"))

# Comment or remove to disable auto screenshotting on error
def pytest_runtest_call(item, __multicall__):
    try:
        __multicall__.execute()
    except Exception as e:
        if hasattr(item.parent.obj, 'driver') or hasattr(item.parent.obj, 'selenium'):
            item.parent.obj.take_named_screenshot('exception')
        raise(e)

def pytest_runtest_makereport(__multicall__, item, call):
    if call.when == "call":
        try:
            assert([] == item.parent.obj.verificationErrors)
        except AssertionError:
            call.excinfo = py.code.ExceptionInfo()

    report = __multicall__.execute()

    item.outcome = report.outcome

    if call.when == "call":
        if hasattr(item.parent.obj, 'config') and item.parent.obj.config.getboolean('SauceLabs', 'ondemand'):
            s = saunter.saucelabs.SauceLabs(item, report)

    return report

def pytest_runtest_teardown(__multicall__, item):
    __multicall__.execute()

    if hasattr(item.parent.obj, 'config') and item.parent.obj.config.getboolean('SauceLabs', 'ondemand'):
        s = saunter.saucelabs.SauceLabs(item)

def pytest_collection_modifyitems(items):
    random.shuffle(items)